package edu.sc.seis.cormorant.event;

import java.sql.SQLException;
import org.omg.CORBA.CompletionStatus;
import edu.iris.Fissures.IfEvent.EventAccess;
import edu.iris.Fissures.IfEvent.EventAccessHelper;
import edu.iris.Fissures.IfEvent.EventChannelFinder;
import edu.iris.Fissures.IfEvent.EventFactory;
import edu.iris.Fissures.IfEvent.EventFinder;
import edu.iris.Fissures.IfEvent.EventFinderPOA;
import edu.iris.Fissures.IfEvent.EventSeqIterHolder;
import edu.sc.seis.fissuresUtil.database.NotFound;
import edu.sc.seis.fissuresUtil.database.event.JDBCCatalog;
import edu.sc.seis.fissuresUtil.database.event.JDBCContributor;
import edu.sc.seis.fissuresUtil.database.event.JDBCEventAccess;
import edu.sc.seis.fissuresUtil.display.MicroSecondTimeRange;
import edu.sc.seis.fissuresUtil.flow.querier.EventFinderQuery;

public class EventFinderImpl extends EventFinderPOA {

    public EventFinderImpl(JDBCEventAccess jdbcEventAccess,
                           JDBCCatalog jdbcCatalog,
                           JDBCContributor jdbcContributor,
                           org.omg.PortableServer.POA poa) {
        this.poa = poa;
        this.jdbcEventAccess = jdbcEventAccess;
        this.jdbcCatalog = jdbcCatalog;
        this.jdbcContributor = jdbcContributor;
    }

    public EventAccess[] query_events(edu.iris.Fissures.Area the_area,
                                      edu.iris.Fissures.Quantity min_depth,
                                      edu.iris.Fissures.Quantity max_depth,
                                      edu.iris.Fissures.TimeRange time_range,
                                      String[] search_types,
                                      float min_magnitude,
                                      float max_magnitude,
                                      String[] catalogs,
                                      String[] contributors,
                                      int seq_max,
                                      EventSeqIterHolder iter) {
        EventAccess[] eventAccess;
        logger.debug("In the method query_events of EventFinder");
        int[] eventIds;
        int counter;
        try {
            logger.debug("The minimum depth is " + min_depth.value);
            logger.debug("The maximum depth is " + max_depth.value);
            logger.debug("The min_magnitude is " + min_magnitude);
            logger.debug("The max_magnitude is " + max_magnitude);
            EventFinderQuery eventFinderQuery = new EventFinderQuery();
            eventFinderQuery.setArea(the_area);
            eventFinderQuery.setMinDepth(min_depth.value);
            eventFinderQuery.setMaxDepth(max_depth.value);
            eventFinderQuery.setMinMag(min_magnitude);
            eventFinderQuery.setMaxMag(max_magnitude);
            eventFinderQuery.setTime(new MicroSecondTimeRange(time_range));
            eventIds = queryEventIds(eventFinderQuery);
            eventAccess = new EventAccess[eventIds.length];
            for(counter = 0; counter < eventIds.length; counter++) {
                new EventAccessImpl(jdbcEventAccess, eventIds[counter]);
                org.omg.CORBA.Object obj = poa.create_reference_with_id(("" + eventIds[counter]).getBytes(),
                                                                        EventAccessHelper.id());
                eventAccess[counter] = EventAccessHelper.narrow(obj);
                if(eventAccess[counter] == null)
                    logger.debug("The returned reference is null");
            }
            return eventAccess;
        } catch(SQLException sqle) {
            logger.error("SQL problem", sqle);
            throw new org.omg.CORBA.UNKNOWN(sqle.toString()); // send
            // exception
            // back to client.
        } catch(Throwable e) {
            logger.error("Generic Exception ", e);
            throw new org.omg.CORBA.UNKNOWN(e.toString(),
                                            1,
                                            CompletionStatus.COMPLETED_NO); // send
            // exceptin
            // back
            // to
            // client.
        }
    }

    protected int[] queryEventIds(EventFinderQuery query) throws SQLException {
        synchronized(jdbcEventAccess.getConnection()) {
            return jdbcEventAccess.query(query);
        }
    }

    public EventAccess[] get_by_name(String name) {
        int[] eventIds;
        int counter;
        EventAccess[] eventAccess;
        try {
            logger.debug("Inside the Function get_byte_name ");
            synchronized(jdbcEventAccess.getConnection()) {
                eventIds = jdbcEventAccess.getByName(name);
            }
            eventAccess = new EventAccess[eventIds.length];
            for(counter = 0; counter < eventIds.length; counter++) {
                /*
                 * EventAccessImpl eventImpl = new
                 * EventAccessImpl(jdbcEventAttr, eventIds[counter].intValue(),
                 * jdbcOrigin, jdbcLocator);
                 */
                org.omg.CORBA.Object obj = poa.create_reference_with_id(("" + eventIds[counter]).getBytes(),
                                                                        EventAccessHelper.id());
                eventAccess[counter] = EventAccessHelper.narrow(obj);
                // System.out.println("Before calling getpreferredorigin int
                // eventfinderimpl");
                // eventAccess[counter].get_preferred_origin();
                if(eventAccess[counter] == null)
                    logger.debug("The returned reference is null");
                String ior = _orb().object_to_string(obj);
                logger.debug("The IOR is " + ior);
            }
            return eventAccess;
        } catch(SQLException sqle) {
            logger.error("SQL Exception ", sqle);
            throw new org.omg.CORBA.UNKNOWN(sqle.toString()); // send
            // exception
            // back to client
        } catch(NotFound nfe) {
            logger.error("Not Found Exception ", nfe);
            throw new org.omg.CORBA.UNKNOWN(nfe.toString()); // send exceptin
            // back to client.
        } catch(Exception e) {
            logger.error("Generic Exception ", e);
            throw new org.omg.CORBA.UNKNOWN(e.toString()); // send exceptin
            // back
            // to client.
        }
    }

    public EventFactory a_factory() {
        /*
         * EventFactory eventFactory = (EventFactory)
         * EventFactoryImpl(jdbcParameterRef, jdbcEventAttr, jdbcFlinnEngdahl,
         * poa);
         */
        // throw new org.omg.CORBA.NO_IMPLEMENT();
        return eventFactory;
    }

    public EventFinder a_finder() {
        return eventFinder;
    }

    public EventChannelFinder a_channel_finder() {
        throw new org.omg.CORBA.NO_IMPLEMENT();
    }

    public String[] known_catalogs() {
        try {
            synchronized(jdbcCatalog.getConnection()) {
                return jdbcCatalog.getAllCatalogs();
            }
        } catch(SQLException sqle) {
            logger.error("sql problem", sqle);
            throw new org.omg.CORBA.UNKNOWN(sqle.toString()); // send
            // exception
            // back to client
        }
    }

    public String[] known_contributors() {
        try {
            synchronized(jdbcContributor.getConnection()) {
                return jdbcContributor.getAll();
            }
        } catch(SQLException sqle) {
            logger.error("sql problem", sqle);
            throw new org.omg.CORBA.UNKNOWN(sqle.toString()); // send
            // exception
            // back to client
        }
    }

    public String[] catalogs_from(String the_contributor) {
        throw new org.omg.CORBA.NO_IMPLEMENT();
    }

    public void setEventFinder(EventFinder eventFinder) {
        this.eventFinder = eventFinder;
    }

    public void setEventFactory(EventFactory eventFactory) {
        this.eventFactory = eventFactory;
    }

    private EventFinder eventFinder = null;

    private EventFactory eventFactory = null;

    protected JDBCEventAccess jdbcEventAccess;

    protected JDBCCatalog jdbcCatalog;

    protected JDBCContributor jdbcContributor;

    org.omg.PortableServer.POA poa;

    private static final org.apache.log4j.Logger logger = org.apache.log4j.Logger.getLogger(EventFinderImpl.class);
} // EventFinderImpl
