package edu.sc.seis.cormorant.nameService;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.util.Properties;
import org.apache.log4j.Logger;
import org.omg.CORBA.ORB;
import org.omg.CosNaming.NamingContextExtHelper;
import org.omg.PortableServer.IdAssignmentPolicyValue;
import org.omg.PortableServer.LifespanPolicyValue;
import org.omg.PortableServer.POA;
import org.omg.PortableServer.POAHelper;
import org.omg.PortableServer.RequestProcessingPolicyValue;
import org.omg.PortableServer.Servant;
import edu.sc.seis.cormorant.Controller;
import edu.sc.seis.fissuresUtil.namingService.FissuresNamingService;

/**
 * @author fenner
 */
public class NameServiceController implements Controller {

    public NameServiceController(Properties confProps,
                                 String serverPropName,
                                 ORB orb) {
        this.serverPropName = serverPropName;
        try {
            this.rootPOA = POAHelper.narrow(orb.resolve_initial_references("RootPOA"));
        } catch(Exception e) {
            logger.fatal("Could not get reference to root POA.");
            throw new RuntimeException("Could not get reference to root POA.",
                                       e);
        }
        org.omg.PortableServer.POAManager manager = rootPOA.the_POAManager();
        try {
            manager.activate();
        } catch(Exception e) {
            logger.fatal("Could not activate POA manager.");
            throw new RuntimeException("Could not activate POA manager.", e);
        }
        org.omg.CORBA.Policy[] policies = new org.omg.CORBA.Policy[3];
        policies[0] = rootPOA.create_id_assignment_policy(IdAssignmentPolicyValue.USER_ID);
        policies[1] = rootPOA.create_lifespan_policy(LifespanPolicyValue.PERSISTENT);
        policies[2] = rootPOA.create_request_processing_policy(RequestProcessingPolicyValue.USE_SERVANT_MANAGER);
        try {
            nsPOA = rootPOA.create_POA("NameServer-POA",
                                       rootPOA.the_POAManager(),
                                       policies);
        } catch(Exception e) {
            logger.fatal("Could not create child POA.");
            throw new RuntimeException("Could not create child POA.", e);
        }
        byte[] oid = (new String("_root").getBytes());
        try {
            nsPOA.create_reference_with_id(oid, NamingContextExtHelper.id());
        } catch(Exception e) {
            logger.fatal("Could not create reference with ID.");
            throw new RuntimeException("Could not create reference with ID.", e);
        }
        this.servant = new NamingContextImpl(confProps,
                                             serverPropName,
                                             orb,
                                             rootPOA);
    }

    public String getServerPropName() {
        return this.serverPropName;
    }

    public void start(ORB orb, FissuresNamingService ns) throws Exception {
        org.omg.CORBA.Object obj = servant._this_object(orb);
        ns.setNameServiceCorbaLoc(obj.toString());
        iorLocFile = new File("NameService.ior");
        BufferedOutputStream bos = new BufferedOutputStream(new FileOutputStream(iorLocFile));
        String property = FissuresNamingService.CORBALOC_PROP + "="
                + ns.getNameServiceCorbaLoc();
        bos.write(property.getBytes());
        logger.info("Name Service IOR placed in - "
                + iorLocFile.getAbsolutePath());
        System.out.println("Name Service IOR placed in - "
                + iorLocFile.getAbsolutePath());
        bos.close();
    }

    public void stop(ORB orb, FissuresNamingService ns) throws Exception {
        if(iorLocFile != null && iorLocFile.exists()) {
            iorLocFile.delete();
        }
    }

    public void destroy() throws Exception {
    // TODO Auto-generated method stub
    }

    public Servant getServant() {
        return this.servant;
    }

    private File iorLocFile;

    private POA rootPOA, nsPOA;

    private Servant servant;

    private String serverPropName;

    private static final Logger logger = Logger.getLogger(NamingContextImpl.class);
}
