package edu.sc.seis.cormorant.network;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;
import org.apache.log4j.Category;
import edu.iris.Fissures.AuditElement;
import edu.iris.Fissures.NotImplemented;
import edu.iris.Fissures.Time;
import edu.iris.Fissures.TimeRange;
import edu.iris.Fissures.IfNetwork.Calibration;
import edu.iris.Fissures.IfNetwork.Channel;
import edu.iris.Fissures.IfNetwork.ChannelId;
import edu.iris.Fissures.IfNetwork.ChannelIdIterHolder;
import edu.iris.Fissures.IfNetwork.ChannelNotFound;
import edu.iris.Fissures.IfNetwork.ComplexNumberErrored;
import edu.iris.Fissures.IfNetwork.ConcreteNetwork;
import edu.iris.Fissures.IfNetwork.ConcreteNetworkAccessPOA;
import edu.iris.Fissures.IfNetwork.Decimation;
import edu.iris.Fissures.IfNetwork.Filter;
import edu.iris.Fissures.IfNetwork.Gain;
import edu.iris.Fissures.IfNetwork.Instrumentation;
import edu.iris.Fissures.IfNetwork.NetworkAttr;
import edu.iris.Fissures.IfNetwork.NetworkId;
import edu.iris.Fissures.IfNetwork.Normalization;
import edu.iris.Fissures.IfNetwork.OrientationRange;
import edu.iris.Fissures.IfNetwork.PoleZeroFilter;
import edu.iris.Fissures.IfNetwork.RecordingStyle;
import edu.iris.Fissures.IfNetwork.Response;
import edu.iris.Fissures.IfNetwork.SamplingRange;
import edu.iris.Fissures.IfNetwork.Sensitivity;
import edu.iris.Fissures.IfNetwork.SiteId;
import edu.iris.Fissures.IfNetwork.SiteNotFound;
import edu.iris.Fissures.IfNetwork.Stage;
import edu.iris.Fissures.IfNetwork.Station;
import edu.iris.Fissures.IfNetwork.StationId;
import edu.iris.Fissures.IfNetwork.StationNotFound;
import edu.iris.Fissures.IfNetwork.TimeCorrection;
import edu.iris.Fissures.IfNetwork.TransferType;
import edu.iris.Fissures.model.QuantityImpl;
import edu.iris.Fissures.model.SamplingImpl;
import edu.iris.Fissures.model.TimeInterval;
import edu.iris.Fissures.model.TimeUtils;
import edu.iris.Fissures.model.UnitImpl;
import edu.iris.Fissures.network.ChannelIdUtil;
import edu.iris.Fissures.network.ClockImpl;
import edu.iris.Fissures.network.DataAcqSysImpl;
import edu.iris.Fissures.network.InstrumentationImpl;
import edu.iris.Fissures.network.NetworkIdUtil;
import edu.iris.Fissures.network.SensorImpl;
import edu.iris.Fissures.network.StationIdUtil;
import edu.sc.seis.fissuresUtil.bag.AreaUtil;
import edu.sc.seis.fissuresUtil.bag.OrientationUtil;
import edu.sc.seis.fissuresUtil.bag.SamplingUtil;
import edu.sc.seis.fissuresUtil.cache.InstrumentationLoader;
import edu.sc.seis.fissuresUtil.database.NotFound;
import edu.sc.seis.fissuresUtil.database.network.JDBCChannel;
import edu.sc.seis.fissuresUtil.database.network.JDBCNetwork;
import edu.sc.seis.fissuresUtil.sac.InvalidResponse;

/**
 * ConcreteNetworkAccessImpl.java Created: Thu Nov 8 15:05:23 2001
 * 
 * @author <a href="mailto: "Philip Crotwell </a>
 * @version
 */
public class ConcreteNetworkAccessImpl extends ConcreteNetworkAccessPOA {

    public ConcreteNetworkAccessImpl(JDBCChannel jdbcChannel,
                                     NetworkId networkId) {
        this.jdbcChannel = jdbcChannel;
        this.jdbcNetwork = jdbcChannel.getNetworkTable();
        this.networkId = networkId;
    }

    //
    // IDL:iris.edu/Fissures/IfNetwork/NetworkAccess/get_attributes:1.0
    //
    /***/
    public synchronized NetworkAttr get_attributes() {
        try {
            return jdbcNetwork.get(networkId);
        } catch(Exception e) {
            logger.error(e.toString(), e);
            throw new org.omg.CORBA.UNKNOWN(e.toString());
        } // end of try-catch
    }

    //
    // IDL:iris.edu/Fissures/IfNetwork/NetworkAccess/retrieve_stations:1.0
    //
    /***/
    public synchronized Station[] retrieve_stations() {
        try {
            return jdbcChannel.getStationTable().getAllStations(networkId);
        } catch(Exception e) {
            logger.error(e.toString(), e);
            throw new org.omg.CORBA.UNKNOWN(e.toString());
        } // end of try-catch
    }

    //
    // IDL:iris.edu/Fissures/IfNetwork/NetworkAccess/retrieve_for_station:1.0
    //
    /***/
    public synchronized Channel[] retrieve_for_station(StationId id) {
        try {
            return jdbcChannel.getAllChannels(id);
        } catch(Exception e) {
            logger.error("retrieve_for_station(" + StationIdUtil.toString(id)
                    + ")", e);
            throw new org.omg.CORBA.UNKNOWN(e.toString());
        } // end of try-catch
    }

    //
    // IDL:iris.edu/Fissures/IfNetwork/NetworkAccess/retrieve_grouping:1.0
    //
    /***/
    public ChannelId[] retrieve_grouping(ChannelId id) throws ChannelNotFound {
        throw new org.omg.CORBA.NO_IMPLEMENT();
    }

    //
    // IDL:iris.edu/Fissures/IfNetwork/NetworkAccess/retrieve_groupings:1.0
    //
    /***/
    public ChannelId[][] retrieve_groupings() {
        throw new org.omg.CORBA.NO_IMPLEMENT();
    }

    //
    // IDL:iris.edu/Fissures/IfNetwork/NetworkAccess/retrieve_channel:1.0
    //
    /***/
    public synchronized Channel retrieve_channel(ChannelId id)
            throws ChannelNotFound {
        logger.debug("retrieve_channel(" + ChannelIdUtil.toString(id) + ")");
        try {
            return jdbcChannel.get(id);
        } catch(NotFound e) {
            logger.error("retrieve_channel(" + ChannelIdUtil.toString(id) + ")",
                         e);
            throw new ChannelNotFound();
        } catch(Exception e) {
            logger.error("retrieve_channel(" + ChannelIdUtil.toString(id) + ")",
                         e);
            throw new org.omg.CORBA.UNKNOWN(e.toString());
        } // end of try-catch
    }

    //
    // IDL:iris.edu/Fissures/IfNetwork/NetworkAccess/retrieve_channels_by_code:1.0
    //
    /***/
    public synchronized Channel[] retrieve_channels_by_code(String station_code,
                                                            String site_code,
                                                            String code)
            throws ChannelNotFound {
        try {
            Channel[] channels = jdbcChannel.getByCode(networkId,
                                                       station_code,
                                                       site_code,
                                                       code);
            return channels;
        } catch(java.sql.SQLException e) {
            logger.error("retrieve_channels_by_code: ", e);
            throw new org.omg.CORBA.UNKNOWN(e.toString());
        } catch(NotFound e) {
            logger.error("retrieve_channels_by_code: ", e);
            throw new org.omg.CORBA.UNKNOWN(e.toString());
        } // end of try-catch
    }

    //
    // IDL:iris.edu/Fissures/IfNetwork/NetworkAccess/locate_channels:1.0
    //
    /***/
    public Channel[] locate_channels(edu.iris.Fissures.Area the_area,
                                     SamplingRange sampling,
                                     OrientationRange orientation) {
        try {
            Channel[] chans = AreaUtil.inArea(the_area, getAllChannels());
            chans = SamplingUtil.inSampling(sampling, chans);
            return OrientationUtil.inOrientation(orientation, chans);
        } catch(Throwable t) {
            logger.error("locate_channels: ", t);
            throw new org.omg.CORBA.UNKNOWN(t.toString());
        }
    }

    //
    // IDL:iris.edu/Fissures/IfNetwork/NetworkAccess/retrieve_instrumentation:1.0
    //
    /***/
    public synchronized Instrumentation retrieve_instrumentation(ChannelId id,
                                                                 Time the_time)
            throws ChannelNotFound {
        InstrumentationImpl inst;
        if(id.network_id.network_code.equals("SP")) {
            // *******************************************
            // WARNING! This is hard coded for the SCEPP PMD sensor
            // and Symmetyric Research digitizer. This is bad coding
            // style and should be changed to get this from the database
            Sensitivity sense = new Sensitivity(1.04178E+09f, 1);
            Stage[] stage = new Stage[2];
            Normalization[] norm = new Normalization[1];
            norm[0] = new Normalization(5.44424E+06f, 1);
            ComplexNumberErrored[] poles = new ComplexNumberErrored[9];
            poles[0] = new ComplexNumberErrored(-22.72205000f,
                                                0,
                                                -47.70198000f,
                                                0);
            poles[1] = new ComplexNumberErrored(-22.72205000f,
                                                0,
                                                47.70198000f,
                                                0);
            poles[2] = new ComplexNumberErrored(-17.77318000f, 0, .00000000f, 0);
            poles[3] = new ComplexNumberErrored(-8.43517300f,
                                                0,
                                                -16.77334000f,
                                                0);
            poles[4] = new ComplexNumberErrored(-8.43517300f,
                                                0,
                                                16.77334000f,
                                                0);
            poles[5] = new ComplexNumberErrored(-.02272769f, 0, -.03170748f, 0);
            poles[6] = new ComplexNumberErrored(-.02272769f, 0, .03170748f, 0);
            poles[7] = new ComplexNumberErrored(-.00638189f, 0, -.00893633f, 0);
            poles[8] = new ComplexNumberErrored(-.00638189f, 0, .00893633f, 0);
            ComplexNumberErrored[] zeros = new ComplexNumberErrored[4];
            zeros[0] = new ComplexNumberErrored(0, 0, 0, 0);
            zeros[1] = zeros[0];
            zeros[2] = zeros[0];
            zeros[3] = zeros[0];
            PoleZeroFilter poleZero = new PoleZeroFilter(poles, zeros);
            Filter[] filter = new Filter[1];
            filter[0] = new Filter();
            filter[0].pole_zero_filter(poleZero);
            stage[0] = new Stage(TransferType.ANALOG,
                                 UnitImpl.METER_PER_SECOND,
                                 UnitImpl.VOLT,
                                 norm,
                                 new Gain(1.49028E+03f, 1),
                                 new Decimation[0],
                                 filter);
            Decimation[] dec = new Decimation[1];
            dec[0] = new Decimation(new SamplingImpl(20,
                                                     new TimeInterval(1,
                                                                      UnitImpl.SECOND)),
                                    1,
                                    0,
                                    new QuantityImpl(0, UnitImpl.SECOND),
                                    new QuantityImpl(0, UnitImpl.SECOND));
            stage[1] = new Stage(TransferType.DIGITAL,
                                 UnitImpl.VOLT,
                                 UnitImpl.COUNT,
                                 new Normalization[0],
                                 new Gain(1.49028E+03f, 1),
                                 dec,
                                 filter);
            Response resp = new Response(sense, stage);
            inst = new InstrumentationImpl(resp,
                                           new TimeRange(id.begin_time,
                                                         TimeUtils.timeUnknown),
                                           new ClockImpl(0,
                                                         "ntp.org",
                                                         "4.0",
                                                         "ntp",
                                                         "ntp"),
                                           new SensorImpl(0,
                                                          "PMD",
                                                          "0",
                                                          "PMD2023",
                                                          .03f,
                                                          50),
                                           new DataAcqSysImpl(0,
                                                              "Symmetric Research",
                                                              "0",
                                                              "par4 revB",
                                                              RecordingStyle.CONTINUOUS));
        } else if(NetworkIdUtil.toStringNoDates(id.network_id).equals("XE05")) {
            Channel chan = retrieve_channel(id);
            if(chan.my_site.comment.contains("CMG40T")) {
                // *******************************************
                // WARNING! This is hard coded for the CMG 40T sensor. This is bad
                // coding
                // style and should be changed to get this from the database
                // poles/zeros from 
                // http://www.passcal.nmt.edu/instrumentation/Sensor/Response/cmg40t.html
                Normalization[] norm = new Normalization[1];
                norm[0] = new Normalization(1, 1);
                ComplexNumberErrored[] poles = new ComplexNumberErrored[2];
                poles[0] = new ComplexNumberErrored(-0.1481f, 0, 0.1481f, 0);
                poles[1] = new ComplexNumberErrored(-0.1481f, 0, -0.1481f, 0);
                ComplexNumberErrored[] zeros = new ComplexNumberErrored[2];
                zeros[0] = new ComplexNumberErrored(0, 0, 0, 0);
                zeros[1] = zeros[0];
                PoleZeroFilter poleZero = new PoleZeroFilter(poles, zeros);
                Filter[] filter = new Filter[] { new Filter() };
                filter[0].pole_zero_filter(poleZero);
                Stage[] stage = new Stage[2];
                stage[0] = new Stage(TransferType.ANALOG,
                                     UnitImpl.METER_PER_SECOND,
                                     UnitImpl.VOLT,
                                     norm,
                                     new Gain(800f, 1),
                                     new Decimation[0],
                                     filter);
                Decimation[] dec = new Decimation[1];
                if(id.channel_code.startsWith("BH")) {
                    dec[0] = new Decimation(new SamplingImpl(40,
                                                             new TimeInterval(1,
                                                                              UnitImpl.SECOND)),
                                            1,
                                            0,
                                            new QuantityImpl(0, UnitImpl.SECOND),
                                            new QuantityImpl(0, UnitImpl.SECOND));
                } else if(id.channel_code.startsWith("LH")) {
                    dec[0] = new Decimation(new SamplingImpl(1,
                                                             new TimeInterval(1,
                                                                              UnitImpl.SECOND)),
                                            1,
                                            0,
                                            new QuantityImpl(0, UnitImpl.SECOND),
                                            new QuantityImpl(0, UnitImpl.SECOND));
                } else {
                    throw new ChannelNotFound();
                }
                stage[1] = new Stage(TransferType.DIGITAL,
                                     UnitImpl.VOLT,
                                     UnitImpl.COUNT,
                                     new Normalization[0],
                                     new Gain(1 / 1.58997e-06f, 1),
                                     dec,
                                     filter);
                // sensitivity will be fixed by repair response below
                Sensitivity sense = new Sensitivity(-1, 1);
                Response resp = new Response(sense, stage);
                inst = new InstrumentationImpl(resp,
                                               new TimeRange(id.begin_time,
                                                             TimeUtils.timeUnknown),
                                               new ClockImpl(0,
                                                             "Reftek",
                                                             "gps",
                                                             "gps",
                                                             "gps"),
                                               new SensorImpl(0,
                                                              "Guralp",
                                                              "0",
                                                              "cmg40t",
                                                              .01f,
                                                              50),
                                               new DataAcqSysImpl(0,
                                                                  "RefTek",
                                                                  "0",
                                                                  "rt130",
                                                                  RecordingStyle.CONTINUOUS));
                return inst;
            }
            // *******************************************
            // WARNING! This is hard coded for the SNEP sensor. This is bad
            // coding
            // style and should be changed to get this from the database
            Normalization[] norm = new Normalization[1];
            norm[0] = new Normalization(5.730E+08f, 1);
            ComplexNumberErrored[] poles = new ComplexNumberErrored[5];
            poles[0] = new ComplexNumberErrored(-3.701E-02f, 0, 3.701E-02f, 0);
            poles[1] = new ComplexNumberErrored(-3.701E-02f, 0, -3.701E-02f, 0);
            poles[2] = new ComplexNumberErrored(-1.131E+03f, 0, 0, 0);
            poles[3] = new ComplexNumberErrored(-1.005E+03f, 0, 0, 0);
            poles[4] = new ComplexNumberErrored(-5.027E+02f, 0, 0, 0);
            ComplexNumberErrored[] zeros = new ComplexNumberErrored[2];
            zeros[0] = new ComplexNumberErrored(0, 0, 0, 0);
            zeros[1] = zeros[0];
            PoleZeroFilter poleZero = new PoleZeroFilter(poles, zeros);
            Filter[] filter = new Filter[1];
            filter[0] = new Filter();
            filter[0].pole_zero_filter(poleZero);
            Stage[] stage = new Stage[2];
            stage[0] = new Stage(TransferType.ANALOG,
                                 UnitImpl.METER_PER_SECOND,
                                 UnitImpl.VOLT,
                                 norm,
                                 new Gain(1500f, 1),
                                 new Decimation[0],
                                 filter);
            Decimation[] dec = new Decimation[1];
            if(id.channel_code.startsWith("BH")) {
                dec[0] = new Decimation(new SamplingImpl(40,
                                                         new TimeInterval(1,
                                                                          UnitImpl.SECOND)),
                                        1,
                                        0,
                                        new QuantityImpl(0, UnitImpl.SECOND),
                                        new QuantityImpl(0, UnitImpl.SECOND));
            } else if(id.channel_code.startsWith("LH")) {
                dec[0] = new Decimation(new SamplingImpl(1,
                                                         new TimeInterval(1,
                                                                          UnitImpl.SECOND)),
                                        1,
                                        0,
                                        new QuantityImpl(0, UnitImpl.SECOND),
                                        new QuantityImpl(0, UnitImpl.SECOND));
            } else {
                throw new ChannelNotFound();
            }
            stage[1] = new Stage(TransferType.DIGITAL,
                                 UnitImpl.VOLT,
                                 UnitImpl.COUNT,
                                 new Normalization[0],
                                 new Gain(1 / 1.58997e-06f, 1),
                                 dec,
                                 filter);
            // sensitivity will be fixed by repair response below
            Sensitivity sense = new Sensitivity(-1, 1);
            Response resp = new Response(sense, stage);
            inst = new InstrumentationImpl(resp,
                                           new TimeRange(id.begin_time,
                                                         TimeUtils.timeUnknown),
                                           new ClockImpl(0,
                                                         "Reftek",
                                                         "gps",
                                                         "gps",
                                                         "gps"),
                                           new SensorImpl(0,
                                                          "Guralp",
                                                          "0",
                                                          "cmg3t",
                                                          .01f,
                                                          50),
                                           new DataAcqSysImpl(0,
                                                              "RefTek",
                                                              "0",
                                                              "rt130",
                                                              RecordingStyle.CONTINUOUS));
        } else {
            throw new ChannelNotFound();
        }
        try {
            InstrumentationLoader.repairResponse(inst.the_response);
        } catch(InvalidResponse e) {
            // should never happen
            throw new RuntimeException(e);
        }
        return inst;
    }

    //
    // IDL:iris.edu/Fissures/IfNetwork/NetworkAccess/retrieve_calibrations:1.0
    //
    /***/
    public Calibration[] retrieve_calibrations(ChannelId id,
                                               edu.iris.Fissures.TimeRange the_time)
            throws ChannelNotFound, edu.iris.Fissures.NotImplemented {
        throw new NotImplemented();
    }

    //
    // IDL:iris.edu/Fissures/IfNetwork/NetworkAccess/retrieve_time_corrections:1.0
    //
    /***/
    public TimeCorrection[] retrieve_time_corrections(ChannelId id,
                                                      edu.iris.Fissures.TimeRange time_range)
            throws ChannelNotFound, edu.iris.Fissures.NotImplemented {
        throw new NotImplemented();
    }

    //
    // IDL:iris.edu/Fissures/IfNetwork/NetworkAccess/retrieve_all_channels:1.0
    //
    /**
     * This returns all channels for this network. The iterator is ignored and
     * all channels are returned in one fell swoop
     */
    public ChannelId[] retrieve_all_channels(int seq_max,
                                             ChannelIdIterHolder iter) {
        try {
            return getIds(getAllChannels());
        } catch(Throwable t) {
            logger.error("retrieve_all_channls: ", t);
            throw new org.omg.CORBA.UNKNOWN(t.toString());
        }
    }

    /**
     * Gets all the channels for the network id of this network access in the db
     * 
     * @throws NotFound
     * @throws SQLException
     */
    public synchronized Channel[] getAllChannels() throws SQLException,
            NotFound {
        Channel[] chans = jdbcChannel.getAllChannels();
        List results = new ArrayList();
        for(int i = 0; i < chans.length; i++) {
            if(NetworkIdUtil.areEqual(chans[i].get_id().network_id, networkId)) {
                results.add(chans[i]);
            }
        }
        return (Channel[])results.toArray(new Channel[results.size()]);
    }

    public static ChannelId[] getIds(Channel[] chans) {
        ChannelId[] chanIds = new ChannelId[chans.length];
        for(int i = 0; i < chans.length; i++) {
            chanIds[i] = chans[i].get_id();
        }
        return chanIds;
    }

    //
    // IDL:iris.edu/Fissures/IfNetwork/NetworkAccess/get_audit_trail_for_channel:1.0
    //
    /***/
    public edu.iris.Fissures.AuditElement[] get_audit_trail_for_channel(ChannelId id)
            throws ChannelNotFound, edu.iris.Fissures.NotImplemented {
        throw new NotImplemented();
    }

    //
    // IDL:iris.edu/Fissures/IfNetwork/ConcreteNetworkAccess/a_writeable:1.0
    //
    /***/
    public ConcreteNetwork a_writeable() {
        throw new org.omg.CORBA.NO_IMPLEMENT();
    }

    //
    // IDL:iris.edu/Fissures/IfNetwork/ConcreteNetworkAccess/get_audit_trail_for_station:1.0
    //
    /***/
    public edu.iris.Fissures.AuditElement[] get_audit_trail_for_station(StationId id)
            throws StationNotFound, edu.iris.Fissures.NotImplemented {
        throw new NotImplemented();
    }

    //
    // IDL:iris.edu/Fissures/IfNetwork/ConcreteNetworkAccess/get_audit_trail_for_site:1.0
    //
    /***/
    public edu.iris.Fissures.AuditElement[] get_audit_trail_for_site(SiteId id)
            throws SiteNotFound, edu.iris.Fissures.NotImplemented {
        throw new NotImplemented();
    }

    //
    // IDL:iris.edu/Fissures/AuditSystemAccess/get_audit_trail:1.0
    //
    /***/
    public AuditElement[] get_audit_trail() throws NotImplemented {
        throw new NotImplemented();
    }

    protected JDBCChannel jdbcChannel;

    protected JDBCNetwork jdbcNetwork;

    protected NetworkId networkId;

    static Category logger = Category.getInstance(ConcreteNetworkAccessImpl.class.getName());
}// ConcreteNetworkAccessImpl
