package edu.sc.seis.cormorant.network;

import java.sql.SQLException;
import org.apache.log4j.Logger;
import edu.iris.Fissures.IfNetwork.Channel;
import edu.iris.Fissures.IfNetwork.ChannelGroupIterHolder;
import edu.iris.Fissures.IfNetwork.ChannelId;
import edu.iris.Fissures.IfNetwork.ChannelIdIterHolder;
import edu.iris.Fissures.IfNetwork.ChannelNotFound;
import edu.iris.Fissures.IfNetwork.NetworkExplorerPOA;
import edu.iris.Fissures.IfNetwork.OrientationRange;
import edu.iris.Fissures.IfNetwork.SamplingRange;
import edu.iris.Fissures.network.ChannelIdUtil;
import edu.sc.seis.fissuresUtil.bag.AreaUtil;
import edu.sc.seis.fissuresUtil.bag.OrientationUtil;
import edu.sc.seis.fissuresUtil.bag.SamplingUtil;
import edu.sc.seis.fissuresUtil.database.NotFound;
import edu.sc.seis.fissuresUtil.database.network.JDBCChannel;
import edu.sc.seis.fissuresUtil.display.DisplayUtils;

/**
 * NetworkExplorerImpl.java Created: Thu Mar 15 11:35:13 2001
 * 
 * @author Philip Crotwell
 * @version
 */
public class NetworkExplorerImpl extends NetworkExplorerPOA {

    public NetworkExplorerImpl(JDBCChannel jdbcChannel) {
        this.jdbcChannel = jdbcChannel;
    }

    //
    // IDL:iris.edu/Fissures/IfNetwork/NetworkExplorer/locate_channels:1.0
    //
    /***/
    public ChannelId[] locate_channels(edu.iris.Fissures.Area the_area,
                                       SamplingRange sampling,
                                       OrientationRange orientation,
                                       int seq_max,
                                       ChannelIdIterHolder iter) {
        try {
            Channel[] chans = AreaUtil.inArea(the_area, jdbcChannel.getAllChannels());
            chans = SamplingUtil.inSampling(sampling, chans);
            return ConcreteNetworkAccessImpl.getIds(OrientationUtil.inOrientation(orientation,
                                                                                  chans));
        } catch(Throwable e) {
            logger.error("locate_channels ", e);
            throw new org.omg.CORBA.UNKNOWN(e.getMessage());
        }
    }

    //
    // IDL:iris.edu/Fissures/IfNetwork/NetworkExplorer/locate_by_sampling:1.0
    //
    /***/
    public ChannelId[][] locate_by_sampling(edu.iris.Fissures.Area the_area,
                                            SamplingRange sampling,
                                            int seq_max,
                                            ChannelGroupIterHolder iter) {
        try {
            Channel[] chans = AreaUtil.inArea(the_area, jdbcChannel.getAllChannels());
            return group(SamplingUtil.inSampling(sampling, chans));
        } catch(Throwable e) {
            logger.error("locate_by_sampling ", e);
            throw new org.omg.CORBA.UNKNOWN(e.getMessage());
        }
    }

    //
    // IDL:iris.edu/Fissures/IfNetwork/NetworkExplorer/locate_all:1.0
    //
    /***/
    public ChannelId[][] locate_all(edu.iris.Fissures.Area the_area,
                                    int seq_max,
                                    ChannelGroupIterHolder iter) {
        try {
            return group(AreaUtil.inArea(the_area, jdbcChannel.getAllChannels()));
        } catch(Throwable e) {
            logger.error("locate_all ", e);
            throw new org.omg.CORBA.UNKNOWN(e.getMessage());
        }
    }

    private static ChannelId[][] group(Channel[] chans) {
        ChannelId[][] chanIds = new ChannelId[chans.length][1];
        for(int i = 0; i < chans.length; i++) {
            chanIds[i][0] = chans[i].get_id();
        }
        return chanIds;
    }

    //
    // IDL:iris.edu/Fissures/IfNetwork/NetworkExplorer/retrieve_channel:1.0
    //
    /***/
    public Channel retrieve_channel(ChannelId a_channel) throws ChannelNotFound {
        try {
            return jdbcChannel.get(a_channel);
        } catch(SQLException e) {
            throw new org.omg.CORBA.UNKNOWN(e.getMessage());
        } catch(NotFound e) {
            throw new ChannelNotFound(e.getMessage(), a_channel);
        }
    }

    //
    // IDL:iris.edu/Fissures/IfNetwork/NetworkExplorer/retrieve_grouping:1.0
    //
    /***/
    public ChannelId[] retrieve_grouping(ChannelId a_channel)
            throws ChannelNotFound {
        try {
            //get them out of the DB...
            ChannelId[] chans = jdbcChannel.getAllChannelIds();
            ChannelId[] out = new ChannelId[3];
            int j = 0;
            for(int i = 0; i < chans.length && j < out.length; i++) {
                if(DisplayUtils.areFriends(a_channel, chans[i])) {
                    out[j] = chans[i];
                    j++;
                }
            }
            if(j == 0) { throw new ChannelNotFound(); }
            return out;
        } catch(SQLException e) {
            logger.error("retrieve_grouping "
                    + ChannelIdUtil.toString(a_channel), e);
            throw new org.omg.CORBA.UNKNOWN(e.getMessage());
        }
    }

    protected JDBCChannel jdbcChannel;

    private static final Logger logger = Logger.getLogger(NetworkExplorerImpl.class);
} // NetworkExplorerImpl
