package edu.sc.seis.cormorant.seismogramServer;

import java.sql.SQLException;
import junit.framework.TestCase;
import org.apache.log4j.BasicConfigurator;
import edu.iris.Fissures.IfNetwork.Channel;
import edu.iris.Fissures.IfSeismogramDC.RequestFilter;
import edu.iris.Fissures.model.MicroSecondDate;
import edu.iris.Fissures.model.TimeInterval;
import edu.iris.Fissures.model.UnitImpl;
import edu.iris.Fissures.network.ChannelIdUtil;
import edu.iris.Fissures.seismogramDC.LocalSeismogramImpl;
import edu.sc.seis.cormorant.seismogram.SodDBSeismogramServer;
import edu.sc.seis.fissuresUtil.chooser.ClockUtil;
import edu.sc.seis.fissuresUtil.database.ConnMgr;
import edu.sc.seis.fissuresUtil.database.seismogram.JDBCSeismogramFiles;
import edu.sc.seis.fissuresUtil.display.SimplePlotUtil;
import edu.sc.seis.fissuresUtil.mockFissures.IfNetwork.MockChannel;
import edu.sc.seis.fissuresUtil.xml.SeismogramFileTypes;

public class SodDBSeismogramServerTest extends TestCase {

    public SodDBSeismogramServerTest() throws SQLException {
        BasicConfigurator.configure();
        chan = MockChannel.createChannel();
        TimeInterval oneDay = new TimeInterval(1, UnitImpl.DAY);
        MicroSecondDate threeDaysAgo = ClockUtil.now()
                .subtract((TimeInterval)oneDay.multiplyBy(3));
        jdbcSeisFiles = new JDBCSeismogramFiles(ConnMgr.createConnection());
        sodDBSeisServer = new SodDBSeismogramServer(jdbcSeisFiles, true);
        for(int i = 0; i < seis.length; i++) {
            seis[i] = SimplePlotUtil.createSpike(threeDaysAgo.add((TimeInterval)oneDay.multiplyBy(i)),
                                                 oneDay,
                                                 50,
                                                 chan.get_id());
            rfs[i] = new RequestFilter(chan.get_id(),
                                       seis[i].begin_time,
                                       seis[i].getEndTime().getFissuresTime());
            jdbcSeisFiles.saveSeismogramToDatabase(chan, seis[i], "unimportant"
                    + i, SeismogramFileTypes.SAC);
        }
    }

    public void testFullDayRequest() throws SQLException {
        for(int i = 0; i < seis.length; i++) {
            RequestFilter[] rfout = sodDBSeisServer.available_data(new RequestFilter[] {rfs[i]});
            assertEquals(1, rfout.length);
            areEqual(rfs[i], rfout[0]);
        }
    }

    private void areEqual(RequestFilter rfin, RequestFilter rfout) {
        assertEquals(new MicroSecondDate(rfin.start_time),
                     new MicroSecondDate(rfout.start_time));
        assertEquals(new MicroSecondDate(rfin.end_time),
                     new MicroSecondDate(rfout.end_time));
        assertTrue(ChannelIdUtil.areEqual(rfin.channel_id, rfout.channel_id));
    }

    public void testDaySpanningRequest() {
        MicroSecondDate startDate = new MicroSecondDate(rfs[0].start_time).add(new TimeInterval(.5,
                                                                                                UnitImpl.DAY));
        MicroSecondDate endDate = startDate.add(new TimeInterval(1,
                                                                 UnitImpl.DAY));
        RequestFilter req = new RequestFilter(rfs[0].channel_id,
                                              startDate.getFissuresTime(),
                                              endDate.getFissuresTime());
        RequestFilter[] rfout = sodDBSeisServer.available_data(new RequestFilter[] {req});
        assertEquals(2, rfout.length);
        assertEquals(startDate, new MicroSecondDate(rfout[0].start_time));
        assertEquals(endDate, new MicroSecondDate(rfout[1].end_time));
    }

    public void testSplitDayRequest() {}

    private SodDBSeismogramServer sodDBSeisServer;

    private JDBCSeismogramFiles jdbcSeisFiles;

    private Channel chan;

    private LocalSeismogramImpl[] seis = new LocalSeismogramImpl[2];

    private RequestFilter[] rfs = new RequestFilter[seis.length];
}
